import { AggregationMapping, FieldMapping } from '@admin-client/reporting-shared';
import { AggregationMappingFormE2EComponent } from '../../../components/aggregation-mapping/aggregation-mapping-form.e2e.component';
import { AggregationMappingE2EComponent } from '../../../components/aggregation-mapping/aggregation-mapping.e2e.component';
import { E2EAggregationMappingHelper } from '../../../helper/aggregation-mapping/aggregation-mapping.helper';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { loginAsDaria } from '../../../support/user-util';

describe('Aggregation Mapping hinzufügen', () => {
  const component: AggregationMappingE2EComponent = new AggregationMappingE2EComponent();
  const formComponent: AggregationMappingFormE2EComponent = new AggregationMappingFormE2EComponent();

  const helper: E2EAggregationMappingHelper = new E2EAggregationMappingHelper();

  const fieldMapping0: FieldMapping = {
    sourcePath: '/path/to/source/a',
    targetPath: '/path/to/target/a',
  };

  const fieldMapping1: FieldMapping = {
    sourcePath: '/path/to/source/b',
    targetPath: '/path/to/target/b',
  };

  const aggregationMapping: AggregationMapping = {
    name: 'Aggregation Mapping',
    formIdentifier: {
      formEngineName: 'formEngineName',
      formId: 'formId',
    },
    mappings: [fieldMapping0, fieldMapping1],
  };

  before(() => {
    loginAsDaria();
  });

  after(() => {
    dropCollections();
  });

  it('should show "Weitere Felder auswerten" button', () => {
    exist(component.getWeitereFelderAuswertenButton());
  });

  it('should show form after button click', () => {
    component.getWeitereFelderAuswertenButton().click();

    exist(formComponent.getRootElement());
  });

  it('should navigate to aggregation mapping on cancel', () => {
    formComponent.getCancelButton().click();

    exist(component.getWeitereFelderAuswertenButton());
  });

  it('should add data field in form', () => {
    component.getWeitereFelderAuswertenButton().click();
    formComponent.getAddFieldButton().click();

    exist(formComponent.getDataFieldInput(1));
  });

  it('should fill out form with two data fields', () => {
    helper.enterForm(aggregationMapping);

    helper.verifyForm(aggregationMapping);
  });

  it('should delete data fields', () => {
    formComponent.getDataFieldDeleteButton(0).click();

    notExist(formComponent.getDataFieldInput(1));
  });

  it('should show aggregation mapping in list after save', () => {
    formComponent.getSaveButton().click();

    helper.verifyAggregationMappingInList(aggregationMapping);
  });
});
