import UserRepresentation from '@keycloak/keycloak-admin-client/lib/defs/userRepresentation';
import {
  BenutzerDeleteDialogE2EComponent,
  BenutzerE2EComponent,
  BenutzerListE2EComponent,
} from '../../components/benutzer/benutzer.e2e.component';
import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { OrganisationsEinheitE2E } from '../../model/organisations-einheit';
import { AdminUserE2E } from '../../model/util';
import { exist, notExist } from '../../support/cypress.util';
import Chainable = Cypress.Chainable;

export class E2EBenutzerExecutor {
  private benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();
  private snackBar: SnackBarE2EComponent = new SnackBarE2EComponent();
  private benutzerDeleteDialog: BenutzerDeleteDialogE2EComponent = new BenutzerDeleteDialogE2EComponent();
  private benutzerListPage: BenutzerListE2EComponent = new BenutzerListE2EComponent();

  public createBenutzer(user: AdminUserE2E): void {
    this.benutzerPage.getBenutzernameInput().type(user.username);
    this.modifyBenutzer(user);
  }

  public modifyBenutzer(user: AdminUserE2E): void {
    this.benutzerPage.getVornameInput().clear().type(user.vorname);
    this.benutzerPage.getNachnameInput().clear().type(user.nachname);
    this.benutzerPage.getMailInput().clear().type(user.email);

    this.benutzerPage.getAdminCheckbox().getRoot()[user.isAdmin ? 'check' : 'uncheck']();
    this.benutzerPage.getUserCheckbox().getRoot()[user.isUser ? 'check' : 'uncheck']();
    this.benutzerPage.getLoeschenCheckbox().getRoot()[user.isLoeschen ? 'check' : 'uncheck']();
    this.benutzerPage.getPostCheckbox().getRoot()[user.isPoststelle ? 'check' : 'uncheck']();

    this.modifyOrganisationsEinheiten(user.organisationseinheiten);
  }

  public modifyOrganisationsEinheiten(organisationseinheiten: OrganisationsEinheitE2E[]): void {
    organisationseinheiten.forEach((name: string) => this.benutzerPage.getOrganisationsEinheitCheckbox(name).click());
  }

  public saveAndCloseSnackbar(): void {
    this.saveBenutzer();
    exist(this.snackBar.getMessage());
    this.snackBar.getCloseButton().click();
    notExist(this.snackBar.getMessage());
    exist(this.benutzerListPage.getList());
  }

  public saveBenutzer(): void {
    this.benutzerPage.getSaveButton().click();
  }

  public deleteBenutzer(): void {
    this.benutzerPage.getDeleteButton().click();
    exist(this.benutzerDeleteDialog.getDeleteButton());
    this.benutzerDeleteDialog.getDeleteButton().click();
    exist(this.benutzerListPage.getList());
  }

  public verifyEmail(userId: string): Chainable<Cypress.Response<void>> {
    return cy.verifyEmail(userId);
  }

  public resetPassword(userId: string, newPassword: string): Chainable<Cypress.Response<void>> {
    return cy.resetPassword(userId, newPassword);
  }

  public findUser(username: string): Chainable<UserRepresentation> {
    return cy.findUser(username);
  }
}
