import UserRepresentation from '@keycloak/keycloak-admin-client/lib/defs/userRepresentation';
import { OrganisationsEinheitE2E } from '../../model/organisations-einheit';
import { AdminUserE2E } from '../../model/util';
import { E2EBenutzerExecutor } from './benutzer.executor';
import { E2EBenutzerNavigator } from './benutzer.navigator';

export class E2EBenutzerHelper {
  private navigator: E2EBenutzerNavigator = new E2EBenutzerNavigator();
  private executor: E2EBenutzerExecutor = new E2EBenutzerExecutor();

  public openBenutzerListPage(): void {
    this.navigator.openBenutzerListPage();
  }

  public openNewBenutzerPage(): void {
    this.navigator.openNewBenutzerPage();
  }

  public addBenutzerAndSave(user: AdminUserE2E): void {
    this.addBenutzer(user);
    this.saveAndCloseSnackbar();
  }

  public addBenutzer(user: AdminUserE2E): void {
    this.createBenutzer(user);
  }

  public editBenutzerAndSave(user: AdminUserE2E): void {
    this.editBenutzer(user);
    this.saveAndCloseSnackbar();
  }

  public editBenutzer(user: AdminUserE2E): void {
    this.modifyBenutzer(user);
  }

  private modifyBenutzer(user: AdminUserE2E): void {
    this.executor.modifyBenutzer(user);
  }

  private createBenutzer(user: AdminUserE2E): void {
    this.executor.createBenutzer(user);
  }

  public editOrganisationsEinheitenAndSave(organisationsEinheiten: OrganisationsEinheitE2E[]): void {
    this.executor.modifyOrganisationsEinheiten(organisationsEinheiten);
    this.saveAndCloseSnackbar();
  }

  private saveAndCloseSnackbar(): void {
    this.executor.saveAndCloseSnackbar();
  }

  public saveBenutzer(): void {
    this.executor.saveBenutzer();
  }

  public deleteBenutzer(userName: string): void {
    this.openBenutzerPage(userName);
    this.executor.deleteBenutzer();
  }

  public openBenutzerPage(userName: string): void {
    this.navigator.openBenutzerPage(userName);
  }

  public activateUser(username: string, newPassword: string): Cypress.Chainable<UserRepresentation> {
    return cy
      .findUser(username)
      .then((userRepresentation: UserRepresentation) => cy.verifyEmail(userRepresentation.id!).then(() => userRepresentation))
      .then((userRepresentation: UserRepresentation) =>
        cy.resetPassword(userRepresentation.id!, newPassword).then(() => userRepresentation),
      );
  }
}
