import { ADMIN_FORMSERVICE } from '@admin-client/shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getElementComponentFromFixtureByCss, mock, Mock, mockGetValue } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { expect } from '@jest/globals';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { AggregationMappingFormService } from '../aggregation-mapping-form.service';
import { AggregationMappingFieldFormComponent } from './aggregation-mapping-field-form/aggregation-mapping-field-form.component';
import { AggregationMappingFieldListFormComponent } from './aggregation-mapping-field-list-form.component';

describe('AggregationMappingFieldListFormComponent', () => {
  let component: AggregationMappingFieldListFormComponent;
  let fixture: ComponentFixture<AggregationMappingFieldListFormComponent>;

  const mappingForm: string = getDataTestIdOf('aggregation-mapping-field-mapping-form-0');

  const formBuilder: FormBuilder = new FormBuilder();

  let formService: Mock<AggregationMappingFormService>;

  beforeEach(async () => {
    const form: FormGroup = formBuilder.group({
      [AggregationMappingFormService.FIELD_MAPPINGS]: formBuilder.array([
        new FormGroup({
          [AggregationMappingFormService.FIELD_MAPPING_SOURCE_PATH]: new FormControl(EMPTY_STRING),
          [AggregationMappingFormService.FIELD_MAPPING_TARGET_PATH]: new FormControl(EMPTY_STRING),
        }),
      ]),
    });

    formService = <any>{
      ...mock(AggregationMappingFormService),
      form,
      addMapping: jest.fn(),
      removeMapping: jest.fn(),
    };

    mockGetValue(
      formService,
      AggregationMappingFormService.FIELD_MAPPINGS,
      form.controls[AggregationMappingFormService.FIELD_MAPPINGS],
    );

    await TestBed.configureTestingModule({
      imports: [
        ReactiveFormsModule,
        AggregationMappingFieldListFormComponent,
        MockComponent(AggregationMappingFieldFormComponent),
      ],
      providers: [
        {
          provide: ADMIN_FORMSERVICE,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AggregationMappingFieldListFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('mapping input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, mappingForm);
      });

      it('should have inputs', () => {
        const mappingComponent: AggregationMappingFieldFormComponent = getElementComponentFromFixtureByCss(fixture, mappingForm);

        expect(mappingComponent.index).toEqual(0);
      });
    });
  });
});
