import { AggregationMappingResource } from '@admin-client/reporting-shared';
import { ADMIN_FORMSERVICE, AdminCancelButtonComponent, AdminSaveButtonComponent, ROUTES } from '@admin-client/shared';
import { createStateResource, EMPTY_STRING, StateResource } from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixtureByType,
  mock,
  Mock,
  MockEvent,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormBuilder, FormControl, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { expect } from '@jest/globals';
import { TextEditorComponent } from '@ods/component';
import { ButtonComponent, PlusIconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { createAggregationMappingResource } from '../../../../../reporting-shared/test/aggregation-mapping';
import { AggregationMappingFieldListFormComponent } from './aggregation-mapping-field-list-form/aggregation-mapping-field-list-form.component';
import { AggregationMappingFormComponent } from './aggregation-mapping-form.component';
import { AggregationMappingFormService } from './aggregation-mapping-form.service';

describe('AggregationMappingFormComponent', () => {
  let component: AggregationMappingFormComponent;
  let fixture: ComponentFixture<AggregationMappingFormComponent>;

  const formEngineNameInputTestId: string = getDataTestIdOf('form-engine-name');
  const formIdInputTestId: string = getDataTestIdOf('form-id');
  const addMappingButton: string = getDataTestIdOf('add-mapping');

  const aggregationMappingStateResource: StateResource<AggregationMappingResource> = createStateResource(
    createAggregationMappingResource(),
  );

  const formBuilder: FormBuilder = new FormBuilder();

  let formService: Mock<AggregationMappingFormService>;

  beforeEach(async () => {
    const form: FormGroup = formBuilder.group({
      [AggregationMappingFormService.FIELD_NAME]: new FormControl(EMPTY_STRING),
      [AggregationMappingFormService.FIELD_FORM_IDENTIFIER]: formBuilder.group({
        [AggregationMappingFormService.FIELD_FORM_ENGINE_NAME]: new FormControl(EMPTY_STRING),
        [AggregationMappingFormService.FIELD_FORM_ID]: new FormControl(EMPTY_STRING),
      }),
    });

    formService = <any>{ ...mock(AggregationMappingFormService), form };
    formService.get = jest.fn().mockReturnValue(of(aggregationMappingStateResource));

    await TestBed.configureTestingModule({
      imports: [
        ReactiveFormsModule,
        AggregationMappingFormComponent,
        MockComponent(TextEditorComponent),
        MockComponent(ButtonComponent),
        MockComponent(PlusIconComponent),
        MockComponent(AdminSaveButtonComponent),
        MockComponent(AdminCancelButtonComponent),
        MockComponent(AggregationMappingFieldListFormComponent),
      ],
    })
      .overrideComponent(AggregationMappingFormComponent, {
        set: {
          providers: [
            {
              provide: AggregationMappingFormService,
              useValue: formService,
            },
            {
              provide: ADMIN_FORMSERVICE,
              useValue: formService,
            },
          ],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(AggregationMappingFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    it('should get aggregation mapping from form service', () => {
      expect(formService.get).toHaveBeenCalled();
    });

    it('should initial values', () => {
      expect(component.aggregationMappingStateResource$).toBeObservable(singleColdCompleted(aggregationMappingStateResource));
    });
  });

  describe('template', () => {
    describe('form engine input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, formEngineNameInputTestId);
      });
    });

    describe('form id input', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, formIdInputTestId);
      });
    });

    describe('add mapping button', () => {
      it('should exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, addMappingButton);
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should call formService', () => {
            fixture.detectChanges();

            dispatchEventFromFixture(fixture, addMappingButton, MockEvent.CLICK);

            expect(formService.addMapping).toHaveBeenCalled();
          });
        });
      });
    });

    describe('save button', () => {
      it('should have link path', () => {
        const comp: AdminSaveButtonComponent = getElementFromFixtureByType(fixture, AdminSaveButtonComponent);

        expect(comp.successLinkPath).toEqual(ROUTES.AGGREGATION_MAPPING);
      });
    });

    describe('cancel button', () => {
      it('should have link path', () => {
        const comp: AdminCancelButtonComponent = getElementFromFixtureByType(fixture, AdminCancelButtonComponent);

        expect(comp.linkPath).toEqual(ROUTES.AGGREGATION_MAPPING);
      });
    });
  });
});
