import { AggregationMappingResource, AggregationMappingService, FieldMapping } from '@admin-client/reporting-shared';
import { AbstractFormService, EMPTY_STRING, isLoaded, StateResource } from '@alfa-client/tech-shared';
import { inject, Injectable } from '@angular/core';
import { FormArray, FormControl, FormGroup, UntypedFormGroup } from '@angular/forms';
import { filter, Observable, tap } from 'rxjs';

@Injectable()
export class AggregationMappingFormService extends AbstractFormService<AggregationMappingResource> {
  public static readonly FIELD_NAME: string = 'name';
  public static readonly FIELD_FORM_IDENTIFIER: string = 'formIdentifier';
  public static readonly FIELD_FORM_ENGINE_NAME: string = 'formEngineName';
  public static readonly FIELD_FORM_ID: string = 'formId';
  public static readonly FIELD_MAPPINGS: string = 'mappings';
  public static readonly FIELD_MAPPING_SOURCE_PATH = 'sourcePath';
  public static readonly FIELD_MAPPING_TARGET_PATH = 'targetPath';

  private readonly aggregationMappingService = inject(AggregationMappingService);

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [AggregationMappingFormService.FIELD_NAME]: new FormControl(EMPTY_STRING),
      [AggregationMappingFormService.FIELD_FORM_IDENTIFIER]: this.formBuilder.group({
        [AggregationMappingFormService.FIELD_FORM_ENGINE_NAME]: new FormControl(EMPTY_STRING),
        [AggregationMappingFormService.FIELD_FORM_ID]: new FormControl(EMPTY_STRING),
      }),
      [AggregationMappingFormService.FIELD_MAPPINGS]: new FormArray([this.createArrayControl()]),
    });
  }

  protected doSubmit(): Observable<StateResource<AggregationMappingResource>> {
    if (this.isPatch()) {
      return this.aggregationMappingService.save(this.getFormValue());
    }
    return this.aggregationMappingService.create(this.getFormValue());
  }

  protected getPathPrefix(): string {
    return EMPTY_STRING;
  }

  public addMapping(): void {
    this.mappings.push(this.createArrayControl());
  }

  private createArrayControl(sourcePath: string = EMPTY_STRING, targetPath: string = EMPTY_STRING): FormGroup {
    return new FormGroup({
      [AggregationMappingFormService.FIELD_MAPPING_SOURCE_PATH]: new FormControl(sourcePath),
      [AggregationMappingFormService.FIELD_MAPPING_TARGET_PATH]: new FormControl(targetPath),
    });
  }

  public removeMapping(index: number): void {
    this.mappings.removeAt(index);
  }

  public get mappings(): FormArray {
    return this.form.controls[AggregationMappingFormService.FIELD_MAPPINGS] as FormArray;
  }

  public get(): Observable<StateResource<AggregationMappingResource>> {
    return this.aggregationMappingService.getByCurrentUrl().pipe(
      filter(isLoaded),
      tap((stateResource: StateResource<AggregationMappingResource>) => this._patchForm(stateResource.resource)),
    );
  }

  _patchForm(value: AggregationMappingResource): void {
    this.patch(value);
    const mappingsFormArray: FormArray = this.form.controls[AggregationMappingFormService.FIELD_MAPPINGS] as FormArray;
    mappingsFormArray.clear();
    value.mappings.forEach((mapping: FieldMapping) =>
      mappingsFormArray.push(this.createArrayControl(mapping.sourcePath, mapping.targetPath)),
    );
  }
}
