/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AggregationMappingListResource, AggregationMappingService } from '@admin-client/reporting-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { expectComponentExistsInTemplate, getElementFromFixtureByType, mock, Mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { RoutingButtonComponent } from '@ods/component';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createAggregationMappingListResource } from '../../../../reporting-shared/test/aggregation-mapping';
import { AggregationMappingListContainerComponent } from './aggregation-mapping-list-container.component';
import { AggregationMappingListComponent } from './aggregation-mapping-list/aggregation-mapping-list.component';

describe('AggregationMappingListContainerComponent', () => {
  let component: AggregationMappingListContainerComponent;
  let fixture: ComponentFixture<AggregationMappingListContainerComponent>;

  let aggregationMappingService: Mock<AggregationMappingService>;

  beforeEach(async () => {
    aggregationMappingService = mock(AggregationMappingService);

    await TestBed.configureTestingModule({
      imports: [
        AggregationMappingListContainerComponent,
        MockComponent(RoutingButtonComponent),
        MockComponent(AggregationMappingListComponent),
      ],
      providers: [
        {
          provide: AggregationMappingService,
          useValue: aggregationMappingService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AggregationMappingListContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on init', () => {
    const stateResource: StateResource<AggregationMappingListResource> = createStateResource(
      createAggregationMappingListResource(),
    );

    beforeEach(() => {
      aggregationMappingService.getList.mockReturnValue(singleCold(stateResource));
    });

    it('should call service to get list', () => {
      component.ngOnInit();

      expect(aggregationMappingService.getList).toHaveBeenCalled();
    });

    it('should assign stateResource', () => {
      component.ngOnInit();

      expect(component.listStateResource$).toBeObservable(singleCold(stateResource));
    });
  });

  describe('on destroy', () => {
    it('should refresh aggregation mapping list', () => {
      component.ngOnDestroy();

      expect(aggregationMappingService.refreshList).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('aggregation mapping list', () => {
      it('should exists', () => {
        expectComponentExistsInTemplate(fixture, AggregationMappingListComponent);
      });

      it('should have inputs', () => {
        const stateResource: StateResource<AggregationMappingListResource> = createStateResource(
          createAggregationMappingListResource(),
        );
        component.listStateResource$ = of(stateResource);
        fixture.detectChanges();

        const comp = getElementFromFixtureByType(fixture, AggregationMappingListComponent);

        expect(comp.aggregationMappingListStateResource).toEqual(stateResource);
      });
    });
  });
});
