import { AggregationMappingListResource } from '@admin-client/reporting-shared';
import {
  createEmptyStateResource,
  createLoadingStateResource,
  createStateResource,
  getEmbeddedResources,
  StateResource,
} from '@alfa-client/tech-shared';
import { expectComponentExistsInTemplate, getElementFromFixtureByType } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { expect } from '@jest/globals';
import { SpinnerComponent } from '@ods/component';
import { ListComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { AggregationMappingListLinkRel } from '../../../../../reporting-shared/src/lib/aggregation-mapping.linkrel';
import { createAggregationMappingListResource } from '../../../../../reporting-shared/test/aggregation-mapping';
import { AggregationMappingListItemComponent } from './aggregation-mapping-list-item/aggregation-mapping-list-item.component';
import { AggregationMappingListComponent } from './aggregation-mapping-list.component';

describe('AggregationMappingListComponent', () => {
  let component: AggregationMappingListComponent;
  let fixture: ComponentFixture<AggregationMappingListComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [
        AggregationMappingListComponent,
        MockComponent(SpinnerComponent),
        MockComponent(ListComponent),
        MockComponent(AggregationMappingListItemComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(AggregationMappingListComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    const aggregationMappingListStateResource: StateResource<AggregationMappingListResource> = createStateResource(
      createAggregationMappingListResource(),
    );

    it('should have initial state', () => {
      expect(component.listStateResource).toEqual(createEmptyStateResource());
      expect(component.aggregationMappings).toEqual([]);
    });

    describe('_update', () => {
      it('should set list state resource', () => {
        component.aggregationMappingListStateResource = aggregationMappingListStateResource;

        expect(component.listStateResource).toEqual(aggregationMappingListStateResource);
      });

      it('should set aggregation mappings if loaded and not nil', () => {
        component.aggregationMappingListStateResource = aggregationMappingListStateResource;

        expect(component.aggregationMappings).toEqual(
          getEmbeddedResources(aggregationMappingListStateResource, AggregationMappingListLinkRel.LIST),
        );
      });

      it('should NOT set aggregation mappings if nil', () => {
        component.aggregationMappingListStateResource = null;

        expect(component.aggregationMappings).toEqual([]);
      });

      it('should NOT set aggregation mappings if loading', () => {
        component.aggregationMappingListStateResource = createLoadingStateResource();

        expect(component.aggregationMappings).toEqual([]);
      });
    });

    describe('set aggregation mapping list state resource', () => {
      it('should update component state', () => {
        component._update = jest.fn();

        component.aggregationMappingListStateResource = aggregationMappingListStateResource;

        expect(component._update).toHaveBeenCalledWith(aggregationMappingListStateResource);
      });
    });
  });

  describe('template', () => {
    const aggregationMappingListStateResource: StateResource<AggregationMappingListResource> = createStateResource(
      createAggregationMappingListResource(),
    );

    beforeEach(() => {
      component.aggregationMappingListStateResource = aggregationMappingListStateResource;
      fixture.detectChanges();
    });

    it('should have spinner with state resource', () => {
      expectComponentExistsInTemplate(fixture, SpinnerComponent);
      const comp = getElementFromFixtureByType(fixture, SpinnerComponent);
      expect(comp.stateResource).toEqual(aggregationMappingListStateResource);
    });

    it('should have list', () => {
      expectComponentExistsInTemplate(fixture, ListComponent);
    });

    it('should have list item with mapping', () => {
      expectComponentExistsInTemplate(fixture, AggregationMappingListItemComponent);
      const comp = getElementFromFixtureByType(fixture, AggregationMappingListItemComponent);
      expect(comp.aggregationMapping).toEqual(
        getEmbeddedResources(aggregationMappingListStateResource, AggregationMappingListLinkRel.LIST)[0],
      );
    });
  });
});
