import {
  AdminOrganisationsEinheitService,
  OrganisationsEinheitDeleteDialogData,
} from '@admin-client/organisations-einheit-shared';
import { createEmptyStateResource, isNotLoading, StateResource } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { DIALOG_DATA } from '@angular/cdk/dialog';
import { AsyncPipe } from '@angular/common';
import { Component, inject } from '@angular/core';
import { Observable, of, tap } from 'rxjs';
import { OrganisationsEinheitDeleteDialogComponent } from './organisations-einheit-delete-dialog/organisations-einheit-delete-dialog.component';

@Component({
  selector: 'admin-organisations-einheit-delete-dialog-container',
  standalone: true,
  imports: [AsyncPipe, OrganisationsEinheitDeleteDialogComponent],
  templateUrl: './organisations-einheit-delete-dialog-container.component.html',
})
export class OrganisationsEinheitDeleteDialogContainerComponent {
  private readonly dialogService = inject(OzgcloudDialogService);
  private readonly dialogData: OrganisationsEinheitDeleteDialogData = inject(DIALOG_DATA);
  private readonly organisationsEinheitService = inject(AdminOrganisationsEinheitService);

  public organisationsEinheitName: string;
  public organisationsEinheitId: string;

  constructor() {
    this.organisationsEinheitName = this.dialogData.organisationsEinheitName;
    this.organisationsEinheitId = this.dialogData.organisationsEinheitId;
  }

  public deleteStateResource$: Observable<StateResource<void>> = of(createEmptyStateResource<void>());

  public closeDialog(): void {
    this.dialogService.closeAll();
  }

  public delete(): void {
    this.deleteStateResource$ = this.organisationsEinheitService
      .delete(this.organisationsEinheitId)
      .pipe(tap((stateResource: StateResource<void>) => this.closeDialogOnDeleteDone(stateResource)));
  }

  private closeDialogOnDeleteDone(stateResource: StateResource<void>): void {
    if (isNotLoading(stateResource)) {
      this.dialogService.closeAll();
    }
  }
}
