/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { mock, Mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { UntypedFormBuilder } from '@angular/forms';
import { Postfach, PostfachResource } from 'libs/admin/postfach-shared/src/lib/postfach.model';
import { PostfachService } from 'libs/admin/postfach-shared/src/lib/postfach.service';
import { createPostfach, createPostfachResource } from 'libs/admin/postfach-shared/src/test/postfach';
import { of } from 'rxjs';
import { PostfachFormService } from './postfach.formservice';

describe('PostfachFormService', () => {
  let formService: PostfachFormService;
  let postfachService: Mock<PostfachService>;

  beforeEach(() => {
    postfachService = mock(PostfachService);

    TestBed.configureTestingModule({
      providers: [PostfachFormService, UntypedFormBuilder, { provide: PostfachService, useValue: postfachService }],
    });

    formService = TestBed.inject(PostfachFormService);
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('submit', () => {
    const postfach: Postfach = createPostfach();

    beforeEach(() => {
      const stateResource: StateResource<PostfachResource> = createStateResource(createPostfachResource());
      postfachService.save.mockReturnValue(of(stateResource));
      postfachService.get.mockReturnValue(of(stateResource));
      formService.form.setValue({
        [PostfachFormService.ASBSENDER_GROUP]: {
          [PostfachFormService.NAME_FIELD]: postfach.absender.name,
          [PostfachFormService.ANSCHRIFT_FIELD]: postfach.absender.anschrift,
          [PostfachFormService.DIENST_FIELD]: postfach.absender.dienst,
          [PostfachFormService.MANDANT_FIELD]: postfach.absender.mandant,
          [PostfachFormService.GEMEINDESCHLUESSEL_FIELD]: postfach.absender.gemeindeschluessel,
        },
        [PostfachFormService.SIGNATUR_FIELD]: postfach.signatur,
      });
    });

    it('should call save on submit', () => {
      formService.submit();

      expect(postfachService.save).toHaveBeenCalledWith(postfach);
    });

    describe('with empty or null absender values', () => {
      const formValueWithAbsender: { [key: string]: any } = {
        [PostfachFormService.ASBSENDER_GROUP]: {
          [PostfachFormService.NAME_FIELD]: '',
          [PostfachFormService.ANSCHRIFT_FIELD]: null,
          [PostfachFormService.DIENST_FIELD]: null,
          [PostfachFormService.MANDANT_FIELD]: '',
          [PostfachFormService.GEMEINDESCHLUESSEL_FIELD]: '',
        },
        [PostfachFormService.SIGNATUR_FIELD]: postfach.signatur,
      };

      beforeEach(() => {
        formService.form.setValue(formValueWithAbsender);
        formService.source = { signatur: postfach.signatur };
      });
      it('should call save without absender', () => {
        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith({ signatur: postfach.signatur });
      });

      it('should call save with absender if any present', () => {
        formValueWithAbsender[PostfachFormService.ASBSENDER_GROUP][PostfachFormService.NAME_FIELD] = 'something';
        formService.form.setValue(formValueWithAbsender);

        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith(formValueWithAbsender);
      });

      it('should call save with absender if valid source', () => {
        formService.source = postfach;

        formService.submit();

        expect(postfachService.save).toHaveBeenCalledWith(formValueWithAbsender);
      });
    });
  });
});
