import { ROUTES } from '@admin-client/shared';
import { NavigationService, RouteData } from '@alfa-client/navigation-shared';
import {
  ApiResourceService,
  createEmptyStateResource,
  createStateResource,
  decodeUrlFromEmbedding,
  ResourceRepository,
  ResourceServiceConfig,
  StateResource,
} from '@alfa-client/tech-shared';
import { UrlSegment } from '@angular/router';
import { iif, map, Observable, of, switchMap } from 'rxjs';
import * as self from './aggregation-mapping-resource.service';
import { AggregationMappingLinkRel } from './aggregation-mapping.linkrel';
import { AggregationMappingResource } from './aggregation-mapping.model';

export class AggregationMappingResourceService extends ApiResourceService<
  AggregationMappingResource,
  AggregationMappingResource
> {}

export function createAggregationMappingResourceService(
  repository: ResourceRepository,
  navigationService: NavigationService,
): AggregationMappingResourceService {
  return new AggregationMappingResourceService(_buildResourceServiceConfig(repository, navigationService), repository);
}

export function _buildResourceServiceConfig(
  repository: ResourceRepository,
  navigationService: NavigationService,
): ResourceServiceConfig<AggregationMappingResource> {
  return {
    resource: self._getResourceByNavigationRoute(repository, navigationService),
    getLinkRel: AggregationMappingLinkRel.SELF,
    edit: { linkRel: AggregationMappingLinkRel.SELF },
    delete: { linkRel: AggregationMappingLinkRel.SELF },
  };
}

export function _getResourceByNavigationRoute(
  repository: ResourceRepository,
  navigationService: NavigationService,
): Observable<StateResource<AggregationMappingResource>> {
  return navigationService
    .getCurrentRouteData()
    .pipe(
      switchMap((route: RouteData) =>
        iif(
          () => self._isAggregationMappingEditUrl(route),
          self._getAggregationMappingResourceByRoute(repository, route),
          of(createEmptyStateResource<AggregationMappingResource>()),
        ),
      ),
    );
}

export function _isAggregationMappingEditUrl(route: RouteData): boolean {
  const urlSegments: UrlSegment[] = route.urlSegments;
  return urlSegments.length === 2 && urlSegments[0].path === ROUTES.AGGREGATION_MAPPING && urlSegments[1].path !== 'neu';
}

export function _getAggregationMappingResourceByRoute(
  repository: ResourceRepository,
  route: RouteData,
): Observable<StateResource<AggregationMappingResource>> {
  const uri: string = decodeUrlFromEmbedding(route.urlSegments[1].path);
  return repository.getResource(uri).pipe(map((resource: AggregationMappingResource) => createStateResource(resource)));
}
