/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createErrorStateResource, createStateResource, StateResource } from '@alfa-client/tech-shared';
import { createCommandResource, createSuccessfullyDoneCommandStateResource } from 'libs/command-shared/test/command';
import { Observable, of } from 'rxjs';
import { CommandResource } from './command.model';
import { tapOnCommandSuccessfullyDone, tapOnFailedCommand } from './command.rxjs.operator';

import { createProblemDetail } from '../../../tech-shared/test/error';
import * as CommandUtil from './command.util';

describe('Command rxjs operator', () => {
  describe('tapOnCommandSuccessfullyDone', () => {
    const command = createStateResource(createCommandResource());
    const commandStateResource$: Observable<StateResource<CommandResource>> = of(command);

    it('should call the execute the runnable if command is successfully done', (done) => {
      jest.spyOn(CommandUtil, 'isSuccessfulDone').mockReturnValue(true);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnCommandSuccessfullyDone(runnable)).subscribe(() => {
        expect(runnable).toHaveBeenCalledWith(command);
        done();
      });
    });

    it('should NOT execute the runnable if command is not done', (done) => {
      jest.spyOn(CommandUtil, 'isSuccessfulDone').mockReturnValue(false);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnCommandSuccessfullyDone(runnable)).subscribe(() => {
        expect(runnable).not.toHaveBeenCalled();
        done();
      });
    });
  });

  describe('tapOnFailedCommand', () => {
    it('should call the execute the runnable if command has failed', (done) => {
      const command = createErrorStateResource(createProblemDetail());
      const commandStateResource$: Observable<StateResource<CommandResource>> = of(command);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnFailedCommand(runnable)).subscribe(() => {
        expect(runnable).toHaveBeenCalledWith(command);
        done();
      });
    });

    it('should NOT execute the runnable if command was successful', (done) => {
      const command = createSuccessfullyDoneCommandStateResource();
      const commandStateResource$: Observable<StateResource<CommandResource>> = of(command);
      const runnable = jest.fn();

      commandStateResource$.pipe(tapOnFailedCommand(runnable)).subscribe(() => {
        expect(runnable).not.toHaveBeenCalled();
        done();
      });
    });
  });
});
