/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { ButtonComponent } from '../button/button.component';
import { ForwardingDirection, ForwardingItemComponent } from './forwarding-item.component';

const label: string = 'Bund für Umwelt und Naturschutz Kreisgruppe Kiel';
const address: string = 'Kaiserstraße 25, 12443 Kiel';

const meta: Meta<ForwardingItemComponent> = {
  title: 'Forwarding item',
  component: ForwardingItemComponent,
  parameters: {
    docs: {
      description: {
        component: 'Information of forwarded item',
      },
    },
  },
  decorators: [
    moduleMetadata({
      imports: [ForwardingItemComponent, ButtonComponent],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<ForwardingItemComponent>;

export const Default: Story = {
  args: {
    label: label,
    address: address,
    direction: ForwardingDirection.TO,
  },
  argTypes: {
    direction: { control: 'select', options: [ForwardingDirection.TO, ForwardingDirection.FROM] },
  },
};

export const WithButton: Story = {
  args: {
    label: label,
    address: address,
  },

  render: (args: ForwardingItemComponent) => ({
    props: args,
    template: `<ods-forwarding-item ${argsToTemplate(args)}>
        <ods-button variant="outline" text="Stelle ändern" />
      </ods-forwarding-item>`,
  }),
};

export const WithCreationInfo: Story = {
  args: {
    label: label,
    address: address,
    direction: ForwardingDirection.FROM,
  },
  render: (args: ForwardingItemComponent) => ({
    props: args,
    template: `<ods-forwarding-item ${argsToTemplate(args)}>
        <p>20. Dez. 09:35</p>
        <p class="text-sm">Karin Wanowski-Müller</p>
      </ods-forwarding-item>`,
  }),
};
