/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { ForwardingService } from '@alfa-client/forwarding-shared';
import { AbstractFormService, StateResource } from '@alfa-client/tech-shared';
import { VorgangResource } from '@alfa-client/vorgang-shared';
import { Injectable } from '@angular/core';
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { Observable } from 'rxjs';

@Injectable()
export class ForwardingByEmailFormService extends AbstractFormService<CommandResource> {
  static readonly FIELD_PATH_PREFIX = 'command.redirectRequest';

  static readonly FIELD_EMAIL = 'email';
  static readonly FIELD_PASSWORD = 'password';

  vorgang: VorgangResource;

  constructor(
    formBuilder: UntypedFormBuilder,
    private forwardingService: ForwardingService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [ForwardingByEmailFormService.FIELD_EMAIL]: new UntypedFormControl(),
      [ForwardingByEmailFormService.FIELD_PASSWORD]: new UntypedFormControl(),
    });
  }

  protected getPathPrefix(): string {
    return ForwardingByEmailFormService.FIELD_PATH_PREFIX;
  }

  public setVorgang(vorgang: VorgangResource): void {
    this.vorgang = vorgang;
  }

  protected doSubmit(): Observable<StateResource<CommandResource>> {
    return this.forwardingService.forwardByEmail(this.vorgang, this.getFormValue());
  }

  patchField(fieldName: string, value: string) {
    this.form.controls[fieldName].patchValue(value);
  }
}
