import { CommandResource, tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { ForwardingService } from '@alfa-client/forwarding-shared';
import { createEmptyStateResource, ResourceRepository, StateResource } from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { createOrganisationEinheitService } from '@alfa-client/zustaendige-stelle';
import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
  ZUSTAENDIGE_STELLE_SERVICE,
} from '@alfa-client/zustaendige-stelle-shared';
import { AsyncPipe, CommonModule } from '@angular/common';
import { Component, inject, OnInit } from '@angular/core';
import { ResourceUri } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { ForwardingDialogComponent } from './forwarding-dialog/forwarding-dialog.component';

@Component({
  selector: 'alfa-forwarding-dialog-container',
  standalone: true,
  imports: [ForwardingDialogComponent, AsyncPipe, CommonModule],
  templateUrl: './forwarding-dialog-container.component.html',
  providers: [
    {
      provide: ZUSTAENDIGE_STELLE_SERVICE,
      useFactory: createOrganisationEinheitService,
      deps: [ResourceRepository, VorgangService],
    },
  ],
})
export class ForwardingDialogContainerComponent implements OnInit {
  private readonly organisationsEinheitService = inject(ZUSTAENDIGE_STELLE_SERVICE) as OrganisationsEinheitService;
  private readonly forwardingService = inject(ForwardingService);
  private readonly dialogService = inject(OzgcloudDialogService);

  public selectedSearchResult$: Observable<OrganisationsEinheitResource>;
  public forwardCommandStateResource$: Observable<StateResource<CommandResource>> =
    of(createEmptyStateResource<CommandResource>());

  ngOnInit(): void {
    this.selectedSearchResult$ = this.organisationsEinheitService.getSelectedResult();
  }

  public forward(targetUri: ResourceUri): void {
    this.forwardCommandStateResource$ = this.forwardingService.forward(targetUri).pipe(
      tapOnCommandSuccessfullyDone(() => {
        this.dialogService.closeAll();
      }),
    );
  }
}
