/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { createEmptyStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatProgressSpinner } from '@angular/material/progress-spinner';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { SpinnerTransparencyComponent } from './spinner-transparency.component';

describe('SpinnerTransparencyComponent', () => {
  let component: SpinnerTransparencyComponent;
  let fixture: ComponentFixture<SpinnerTransparencyComponent>;

  const spinnerSelector: string = getDataTestClassOf('spinner-transparency');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [MatIcon, MatProgressSpinner, SpinnerTransparencyComponent, SpinnerComponent],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(SpinnerTransparencyComponent);
    component = fixture.componentInstance;

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('spinner', () => {
    it('should NOT be visible if NOT loading and already loaded', () => {
      component.stateResource = { ...createEmptyStateResource(false), loaded: true };
      fixture.detectChanges();

      const spinner = getElementFromFixture(fixture, spinnerSelector);

      expect(spinner).not.toBeInstanceOf(HTMLElement);
    });

    it('should be visible if loading', () => {
      component.stateResource = createEmptyStateResource(true);
      fixture.detectChanges();

      const spinner = getElementFromFixture(fixture, spinnerSelector);

      expect(spinner).toBeInstanceOf(HTMLElement);
    });
  });
});
