/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { formatFullDate, isParsableToDate, joinAndTrim } from '@alfa-client/tech-shared';
import { Antragsteller } from '@alfa-client/vorgang-shared';
import { Component, Input } from '@angular/core';
import { isEmpty, isNull } from 'lodash-es';

@Component({
  selector: 'alfa-vorgang-detail-antragsteller',
  templateUrl: './vorgang-detail-antragsteller.component.html',
})
export class VorgangDetailAntragstellerComponent {
  private readonly TRIM_TO_ONE_SPACE = /\s{2,}/g;
  public antragstellerExists: boolean;
  public antragstellerData: Antragsteller;
  public name: string;
  public antragstellerStrasseHausnummer: string;
  public antragstellerPlzOrt: string;
  public geburt: string;
  public geburtsdatum: string;
  public isFirma: boolean;

  @Input() set antragsteller(antragsteller: Antragsteller) {
    this.antragstellerExists = this.getAntragstellerExists(antragsteller);
    if (!this.antragstellerExists) {
      return;
    }
    this.antragstellerData = antragsteller;
    this.isFirma = Boolean(antragsteller.firmaName);
    this.setName(antragsteller);
    this.setGeburtsInfo(antragsteller);
    this.setAdresse(antragsteller, this.isFirma);
  }

  getAntragstellerExists(antragsteller: Antragsteller): boolean {
    if (isNull(antragsteller)) {
      return false;
    }
    if (Object.values(antragsteller).every((value) => isEmpty(value))) {
      return false;
    }
    return true;
  }

  formatGeburtsdatum(geburtsdatum: string): string {
    if (isParsableToDate(geburtsdatum)) {
      return formatFullDate(new Date(geburtsdatum));
    }
    return geburtsdatum;
  }

  private setAdresse(antragsteller: Antragsteller, isFirma: boolean): void {
    this.antragstellerStrasseHausnummer = joinAndTrim([
      antragsteller.strasse,
      antragsteller.hausnummer,
    ]);
    this.antragstellerPlzOrt = joinAndTrim([antragsteller.plz, antragsteller.ort]);
  }

  private setName(antragsteller: Antragsteller): void {
    this.name = joinAndTrim([
      antragsteller.anrede,
      antragsteller.vorname,
      antragsteller.nachname,
    ]).replace(this.TRIM_TO_ONE_SPACE, ' ');
  }

  private setGeburtsInfo(antragsteller: Antragsteller): void {
    this.geburtsdatum = this.formatGeburtsdatum(antragsteller.geburtsdatum);
    this.geburt =
      this.geburtsdatum && antragsteller.geburtsort ?
        `${this.geburtsdatum} in ${antragsteller.geburtsort}`
      : this.geburtsdatum || antragsteller.geburtsort;
  }
}
