/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { LoeschAnforderungService } from '@alfa-client/loesch-anforderung-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import { isResourceNotFoundError, StateResource } from '@alfa-client/tech-shared';
import { Messages, SnackBarService } from '@alfa-client/ui';
import { VorgangCommandService, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Component, OnInit } from '@angular/core';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-vorgang-detail-page',
  templateUrl: './vorgang-detail-page.component.html',
  styleUrls: ['./vorgang-detail-page.component.scss'],
})
export class VorgangDetailPageComponent implements OnInit {
  vorgangStateResource$: Observable<StateResource<VorgangWithEingangResource>>;
  revokeCommandStateResource$: Observable<StateResource<CommandResource>>;
  vorgangLoeschenCommandStateResource$: Observable<StateResource<CommandResource>>;

  constructor(
    private vorgangService: VorgangService,
    private vorgangCommandService: VorgangCommandService,
    private loeschAnforderungService: LoeschAnforderungService,
    public snackbarService: SnackBarService,
    public navigationService: NavigationService,
  ) {}

  ngOnInit(): void {
    this.vorgangStateResource$ = this.vorgangService
      .getVorgangWithEingang()
      .pipe(tap((resource: StateResource<VorgangWithEingangResource>) => this.handleApiError(resource)));
    this.revokeCommandStateResource$ = this.vorgangCommandService.getRevokeCommand();
    this.vorgangLoeschenCommandStateResource$ = this.loeschAnforderungService.getEndgueltigLoeschenCommand();
  }

  handleApiError(resource: StateResource<VorgangWithEingangResource>) {
    if (isResourceNotFoundError(resource.error)) {
      this.snackbarService.showInfo(Messages.HTTP_STATUS_VORGANG_NOT_FOUND);
      this.navigationService.navigateToVorgangList();
    }
  }
}
